﻿using System.Collections.Generic;
using System.Text.Json;
using System.Threading.Tasks;
using ExternalServiceV3.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;

namespace ExternalServiceV3.Controllers
{
   [Route("api/3.5/permissions")]
   [ApiController]
   public class PermissionsController : ControllerBase
   {
      private readonly ILogger<PermissionsController> _logger;
      private readonly DataAccess _dataAccess;

      public PermissionsController(ILogger<PermissionsController> logger, DataAccess dataAccess)
      {
         _logger = logger;
         _dataAccess = dataAccess;
      }

      [HttpGet]
      [HeaderAuthorization]
      public async Task<ReaderPermissions> Get(string userId, string token)
      {
         _logger.LogDebug("GET PERMISSIONS ENDPOINT: userId={0} token={1}", userId, token);

         ReaderPermissions results;
         if (string.IsNullOrEmpty(token))
         {
            results = await _dataAccess.GetReaderPermissionsAsync(userId).ConfigureAwait(false);  
         }
         else
         {
            results = await _dataAccess.GetReaderPermissionsByTokenAsync(token).ConfigureAwait(false);
         }
          

         _logger.LogDebug("Get permissions ends with: {Results}", JsonSerializer.Serialize(results));

         return results;
      }
   }
}