﻿using Microsoft.AspNetCore.Mvc;
using ExternalServiceV3.Attributes;
using ExternalServiceV3.Models;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;

namespace ExternalServiceV3.Controllers
{
   [ApiController]
   public class ReaderController : ControllerBase
   {
      private readonly ILogger<ReaderController> _logger;
      private readonly DataAccess _dataAccess;

      public ReaderController(ILogger<ReaderController> logger, DataAccess dataAccess)
      {
         _logger = logger;
         _dataAccess = dataAccess;
      }

      /// <summary>
      /// Retrieve paged sets of readers
      /// </summary>
      /// <param name="page"></param>
      /// <param name="filter"></param>
      /// <param name="sort"></param>
      /// <returns></returns>
      [Route("api/3.5/readers")]
      [BindJson(typeof(Page), "page")]
      [BindJson(typeof(Filter), "filter")]
      [BindJson(typeof(Sort), "sort")]
      [HeaderAuthorization]
      public async Task<PagedList<Reader>> Get([FromQuery]Page page, [FromQuery]Filter filter, [FromQuery]Sort sort)
      {
         _logger.LogDebug("GET READERS ENDPOINT");

         var result = await _dataAccess.GetReadersAsync(page, filter, sort);

         _logger.LogDebug("Results: {Results}", JsonSerializer.Serialize(result));
         _logger.LogDebug("Get readers ends");

         return result;
      }
   }
}