﻿using ExternalServiceV3.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Net.Http.Json;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Timer = System.Timers.Timer;

namespace ExternalServiceV3
{
   public class DataAccess
   {
      private readonly ILogger<DataAccess> _logger;
      private readonly IHttpClientFactory _httpClientFactory;
      private static IReadOnlyList<DataReader> _data = Array.Empty<DataReader>();

      private readonly string ApiBaseUrl;
      private readonly int ApiRefreshIntervalInMinutes;

      public DataAccess(ILogger<DataAccess> logger, IConfiguration configuration, IHttpClientFactory httpClientFactory)
      {
         _logger = logger;
         _httpClientFactory = httpClientFactory;
         ApiBaseUrl = configuration["RemoteApi:BaseUrl"];
         ApiRefreshIntervalInMinutes = configuration.GetSection("RemoteApi:RefreshIntervalInMinutes").Get<int>();

         UpdateData().GetAwaiter().GetResult();
         
         _logger.LogDebug($"Starting Timer every {ApiRefreshIntervalInMinutes} minutes");
         var refreshTimer = new Timer(ApiRefreshIntervalInMinutes * 60 * 1000);
         refreshTimer.Enabled = true;
         refreshTimer.Elapsed += T_Elapsed;
      }

      private void T_Elapsed(object sender, ElapsedEventArgs e)
      {
         UpdateData();
      }
      
      private async Task UpdateData()
      {
         _logger.LogDebug("UpdateData() Started");
         
         //update this code to get all data from your system
         var data = new List<DataReader>();
         for (int i = 1; i <= 7; i++)
            data.Add(new DataReader() { Id = i.ToString(), Username = $"user{i}@domain.com", Password = "other_password", Token = null, PurchaseRecords = null });

         data.Add(
            new DataReader() 
            { 
               Id = "8", 
               Username = "user8@domain.com", 
               Password = "test123", 
               Token = "3BBD5CC2-954A-40A1-8653-C14D8B4B4824", 
               PurchaseRecords = new List<DataPurchaseRecord>() 
               { 
                  new DataPurchaseRecord() 
                  {
                     Product = new DataProduct() { ExternalKey = "ValidExternalKey"},
                     OfflineDays = 7, 
                     BrowserLimit = 2, PdfLimit = 2 
                  } 
               } 
            });

         for (int i = 9; i <= 99; i++)
            data.Add(new DataReader() { Id = i.ToString(), Username = $"user{i}@domain.com", Password = "other_password", Token = null, PurchaseRecords = null });
         
         if (data.Count == 0)
         {
            _logger.LogWarning("UpdateData() Ended by not updating _data, due to result having 0 results.");
            return;
         }
         
         Interlocked.Exchange(ref _data, data);
         _logger.LogDebug($"UpdateData() Ended by updating _data, item count {_data.Count}");
      }

      public async Task<Reader> GetReaderByUsernameAndPasswordAsync(string username, string password, bool caseSensitivePassword)
      {
         //update this code to talk to your system asynchronously
         
         //Please remove this comments for clients code
         //if you need to use http request to get data you can use IHttpClientFactory 
         //I added 4 base methods POST/PUT/GET/DELETE
         //You can use it like here:
         //return await GetAsync<Reader>(new Uri(ApiBaseUrl));
         
         return ConvertDataReaderToReader(
            _data.FirstOrDefault(x => x.Username.Equals(username, StringComparison.InvariantCultureIgnoreCase) 
            && (x.Password.Equals(password, caseSensitivePassword ? StringComparison.InvariantCulture : StringComparison.InvariantCultureIgnoreCase)
            )));
      }

      public async Task<Reader> GetReaderByUsernameAsync(string username)
      {
         //update this code to talk to your system asynchronously
         return ConvertDataReaderToReader(_data.FirstOrDefault(x => x.Username.Equals(username, StringComparison.InvariantCultureIgnoreCase)));
      }

      public async Task<Reader> GetReaderByIdAsync(string id)
      {
         //update this code to talk to your system asynchronously
         return ConvertDataReaderToReader(_data.FirstOrDefault(x => x.Id.Equals(id, StringComparison.InvariantCultureIgnoreCase)));
      }

      public async Task<Reader> GetReaderByTokenAsync(string token)
      {
         //update this code to talk to your system asynchronously
         return ConvertDataReaderToReader(_data.FirstOrDefault(x => token.Equals(x.Token, StringComparison.InvariantCultureIgnoreCase)));
      }

      public async Task<AuthenticationResult> GetAuthenticationAsync(string username, string password, bool caseSensitivePassword, DocumentContext document)
      {
         //update this code to talk to your system asynchronously
         var dataReader = 
            _data.FirstOrDefault(x => x.Username.Equals(username, StringComparison.InvariantCultureIgnoreCase)
            && (x.Password.Equals(password, caseSensitivePassword ? StringComparison.InvariantCulture : StringComparison.InvariantCultureIgnoreCase)
            ));

         if (dataReader == null) 
         {
            //user found but incorrect password
            if (_data.FirstOrDefault(x => x.Username.Equals(username, StringComparison.InvariantCultureIgnoreCase)) != null) 
            {
               return Helpers.GenerateUnsuccessfulAuthenticationResult(
                  "Your login credentials appear to be incorrect. Please try again and ensure the 'Caps Lock' is off (bw5).",
                  new Reader() { Username = username });
            }
         }
         
         return GetAuthenticationHelper(dataReader, document, null, username);
      }

      public async Task<AuthenticationResult> GetAuthenticationAsync(string username, DocumentContext document)
      {
         //update this code to talk to your system asynchronously
         var dataReader =
            _data.FirstOrDefault(x => x.Username.Equals(username, StringComparison.InvariantCultureIgnoreCase));

         return GetAuthenticationHelper(dataReader, document, null, username);
      }

      public async Task<AuthenticationResult> GetAuthenticationByIdAsync(string id, DocumentContext document)
      {
         //update this code to talk to your system asynchronously
         var dataReader =
            _data.FirstOrDefault(x => x.Id.Equals(id, StringComparison.InvariantCultureIgnoreCase));

         return GetAuthenticationHelper(dataReader, document, id, null);
      }

      public async Task<AuthenticationResult> GetAuthenticationByTokenAsync(string token, DocumentContext document)
      {
         //update this code to talk to your system asynchronously
         var dataReader =
            _data.FirstOrDefault(x => token.Equals(x.Token, StringComparison.InvariantCultureIgnoreCase));

         return GetAuthenticationHelper(dataReader, document, null, token);
      }
      
      /// <summary>
      /// Depending on the parameters, deduce if the Vitrium Authenticate endpoint should receive a success or unsuccessful result.
      /// If unsuccessful, provide an appropriate message.
      /// </summary>
      /// <param name="dataReader"></param>
      /// <param name="document"></param>
      /// <param name="userid">Only used if dataReader is null</param>
      /// <param name="username">Only used if dataReader is null</param>
      /// <returns></returns>
      private AuthenticationResult GetAuthenticationHelper(DataReader dataReader, DocumentContext document,
         string userid = null, string username = null)
      {
         if (dataReader == null)
         {
            return Helpers.GenerateUnsuccessfulAuthenticationResult(
               "Your account could not be found in the system. Make sure you are entering the correct details (3yq)", 
               new Reader() { Id = userid, Username = username });
         }

         if (dataReader.PurchaseRecords != null)
         {
            var dataPurchaseRecord = dataReader.PurchaseRecords.FirstOrDefault(x => x.Product.ExternalKey == document.ExternalKey);
            if (dataPurchaseRecord != null)
               return Helpers.GenerateSuccessfulAuthenticationResult(ConvertDataReaderToReader(dataReader), ConvertDataPurchaseRecordToAccessPolicy(dataPurchaseRecord));
         }

         return Helpers.GenerateUnsuccessfulAuthenticationResult("You do not have permission to view this content (w29)", ConvertDataReaderToReader(dataReader));
      }

      public async Task<ReaderPermissions> GetReaderPermissionsAsync(string id)
      {
         //update this code to talk to your system asynchronously
         var dataReader = _data.FirstOrDefault(x => x.Id.Equals(id, StringComparison.InvariantCultureIgnoreCase));
         if (dataReader?.PurchaseRecords != null)
         {
            var externalKeys = dataReader.PurchaseRecords.Select(x => x.Product.ExternalKey).ToList();
            return new ReaderPermissions { DocExternalKeys = externalKeys };
         }
         return new ReaderPermissions { DocExternalKeys = null };
      }
      
      public Task<ReaderPermissions> GetReaderPermissionsByTokenAsync(string token)
      {
         //update this code to talk to your system asynchronously
         var dataReader = _data.FirstOrDefault(x => string.Equals(x.Token, token, StringComparison.InvariantCultureIgnoreCase));
         if (dataReader?.PurchaseRecords != null)
         {
            var externalKeys = dataReader.PurchaseRecords.Select(x => x.Product.ExternalKey).ToList();
            return Task.FromResult(new ReaderPermissions { DocExternalKeys = externalKeys });
         }
         return Task.FromResult(new ReaderPermissions { DocExternalKeys = null });
      }

      public async Task<PagedList<Reader>> GetReadersAsync(Page page, Filter filter, Sort sort)
      {
         //update this code to talk to your system asynchronously
         IEnumerable<DataReader> results = _data.ToList();

         if (filter != null)
         {
            _logger.LogDebug("filter: {Filter}", JsonSerializer.Serialize(filter));
            results = results.Where(r => r.Username.ToLower().Contains(filter.contains.ToLower()));
         }

         int totalRecords = results.Count();

         if (page != null)
         {
            _logger.LogDebug("page: {Page}", JsonSerializer.Serialize(page));
            results = results.Skip((page.index - 1) * page.size).Take(page.size);
         }

         if (sort != null)
         {
            _logger.LogDebug("sort: {Sorting}", JsonSerializer.Serialize(sort));
            results = sort.username == (int)SortingOrder.Ascending ? results.OrderBy(r => r.Username) : results.OrderByDescending(r => r.Username);
         }

         var result = new PagedList<Reader>
         {
            Results = results.Select(x => ConvertDataReaderToReader(x)),
            TotalRecords = totalRecords
         };

         return result;
      }

      private static Reader ConvertDataReaderToReader(DataReader dr)
      {
         if (dr == null)
            return null;

         return new Reader
         {
            Id = dr.Id,
            Username = dr.Username,
            IsActive = true,
            CreatedOn = DateTime.Now.ToString("yyyy-MM-dd"),
         };
      }

      private static AccessPolicy ConvertDataPurchaseRecordToAccessPolicy(DataPurchaseRecord pr)
      {
         if (pr == null)
            return null;

         return new AccessPolicy
         {
            OfflineDurationinDays = pr.OfflineDays,
            ComputersMax = AccessPolicy.NotSet,
            PdfLimit = pr.PdfLimit,
            BrowserLimit = pr.BrowserLimit,
         };
      }
      
      private async Task<T> PostAsync<T>(Uri uri, HttpContent content)
      {
         var result = await _httpClientFactory.CreateClient().PostAsync(uri, content);
         result.EnsureSuccessStatusCode();
         return await result.Content.ReadFromJsonAsync<T>();
      }
      
      private async Task<T> PutAsync<T>(Uri uri, HttpContent content)
      {
         var result = await _httpClientFactory.CreateClient().PutAsync(uri, content);
         result.EnsureSuccessStatusCode();
         return await result.Content.ReadFromJsonAsync<T>();
      }
      
      private async Task<T> GetAsync<T>(Uri uri)
      {
         var result = await _httpClientFactory.CreateClient().GetAsync(uri);
         result.EnsureSuccessStatusCode();
         return await result.Content.ReadFromJsonAsync<T>();
      }
      
      private async Task<T> GetAsync<T>(AuthenticationHeaderValue header, Uri uri)
      {
         var httpRequestMessage = new HttpRequestMessage
         {
            Method = HttpMethod.Get,
            RequestUri = uri
         };
         
         httpRequestMessage.Headers.Authorization = header;
         var result = await _httpClientFactory.CreateClient().SendAsync(httpRequestMessage);
         result.EnsureSuccessStatusCode();
         return await result.Content.ReadFromJsonAsync<T>();
      }
      
      private async Task<bool> DeleteAsync(Uri uri)
      {
         var result = await _httpClientFactory.CreateClient().DeleteAsync(uri);
         return result.IsSuccessStatusCode;
      }
   }

   public class DataReader
   {
      public string Id { get; init; }
      public string Username { get; init; }
      public string Password { get; init; }
      public string Token { get; init; }
      public List<DataPurchaseRecord> PurchaseRecords { get; init; }
   }

   public class DataPurchaseRecord
   {
      public int? OfflineDays { get; init; }
      public int? BrowserLimit { get; init; }
      public int? PdfLimit { get; init; }
      public DataProduct Product { get; init; }
   }

   public class DataProduct
   {
      public string ExternalKey { get; init; }
   }
}
