﻿using ExternalServiceV3.Models;
using System.Collections.Generic;

namespace ExternalServiceV3
{
   public static class Helpers
   {
      /// <summary>
      /// This helper method creates an unsuccessful AuthenticationResult for the Vitrium Authenticate endpoint
      /// </summary>
      /// <param name="message"></param>
      /// <param name="reader"></param>
      /// <returns></returns>
      public static AuthenticationResult GenerateUnsuccessfulAuthenticationResult(string message, Reader reader = null)
      {
         return new AuthenticationResult
         {
            Succeed = false,
            UserId = reader?.Id,
            Username = reader?.Username,
            Message = message
         };
      }

      /// <summary>
      /// This helper method creates a successful AuthenticationResult for the Vitrium Authenticate endpoint
      /// </summary>
      /// <param name="reader"></param>
      /// <param name="accessPolicy"></param>
      /// <returns></returns>
      public static AuthenticationResult GenerateSuccessfulAuthenticationResult(Reader reader, AccessPolicy accessPolicy = null)
      {
         if (reader == null)
            return new AuthenticationResult { Succeed = false };

         return new AuthenticationResult
         {
            Succeed = true,
            Policy = accessPolicy,
            UserId = reader.Id,
            Username = reader.Username,
            WatermarkTokens = new Dictionary<string, string>() { { "_custom_", "Custom Watermark for " + reader.Username + "." } }
         };
      }
   }
}
