﻿using System;
using System.Collections.Generic;

namespace ExternalServiceV3.Models
{
   public class AuthenticationRequest
   {
      public string Username { get; set; }
      public string Id { get; set; }
      public string Password { get; set; }
      public string HashingKey { get; set; }
      public string HashingVersion { get; set; }
      public bool CaseSensitivePassword { get; set; }
      public string Token { get; set; }
      public string Type { get; set; }
      
      public DocumentContext Document { get; set; }
      public UserClientMetadata UserClient { get; set; }
   }
   
   public class DocumentMetadata
   {
      public string ContentType { get; set; }

      public string FileName { get; set; }

      public string VersionName { get; set; }

      public List<WatermarkTemplate> UserSpecificWatermarkTemplates { get; set; }
   }
   
   public class FolderMetadata
   {
      public string Id { get; set; }
      public string Name { get; set; }
      public string PortalFolderName { get; set; }
      public string ExternalKey { get; set; }
      public string CustomField { get; set; }
   }

   public class WatermarkTemplate
   {
      public Guid Id { get; set; }

      public string Name { get; set; }

      public string TextTemplate { get; set; }
   }

   public class DocumentStatus
   {
      public bool IsActive { get; set; }

      public bool IsMostRecentVersion { get; set; }

      public bool IsMostRecentVersionActive { get; set; }
   };

   public class DocumentContext
   {
      /// <summary>
      /// Folder id paths 
      /// </summary>
      public List<Guid> FolderPath { get; set; }

      /// <summary>
      /// This field contains DocumentId that is shared between all versions of same document
      /// </summary>
      public Guid DocumentId { get; set; }

      /// <summary>
      /// VersionId is unique between all revisons of same document
      /// </summary>
      public Guid VersionId { get; set; }

      /// <summary>
      /// DocCode is unique between all revisons of same document
      /// </summary>
      public string DocCode { get; set; }

      public DocumentMetadata Metadata { get; set; }

      public string ExternalKey { get; set; }

      public DocumentStatus Status { get; set; }
      
      public string Alias { get; set; }
      
      public List<FolderMetadata> Folders { get; set; }
   }

   public class UserClientMetadata
   {
      public string AppName { get; set; }

      public string AppVersion { get; set; }

      public string Platform { get; set; }
     
      public string OperatingSystem { get; set; }

      public string DeviceName { get; set; }

      public string DeviceId { get; set; }

      public bool HasOfflineAccess { get; set; }

      public string InjectVersion { get; set; }

      public string IpAddress { get; set; }

      public string Language { get; set; }

      public bool OutOfBrowser { get; set; }

      public string ServerUrl { get; set; }
   }
}
