using System.Threading.Tasks;
using ExternalServiceV3;
using ExternalServiceV3.Controllers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ExternalServiceV3.Models;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;

namespace FunctionalTests
{
   [TestClass]
   public class ReaderControllerTests: BaseTest
   {
      [TestMethod]
      public async Task ReaderSearch_Test()
      {
         var controller = new ReaderController(ServiceProvider.GetService<ILogger<ReaderController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Get(
            new Page() { index = 1, size = 20 },
            new Filter() { contains = "" },
            new Sort() { username = 1 }
            );

         Assert.AreEqual(99, results.TotalRecords);
      }

      [TestMethod]
      public async Task ReaderSearch2_Test()
      {
         var controller = new ReaderController(ServiceProvider.GetService<ILogger<ReaderController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Get(
            new Page() { index = 1, size = 20 },
            new Filter() { contains = "user1" },
            new Sort() { username = 1 }
            );

         Assert.AreEqual(11, results.TotalRecords);
      }

      [TestMethod]
      public async Task ReaderSearch_NoResult_Test()
      {
         var controller = new ReaderController(ServiceProvider.GetService<ILogger<ReaderController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Get(
            new Page() { index = 1, size = 20 },
            new Filter() { contains = "user_does_not_exist" },
            new Sort() { username = 1 }
            );

         Assert.AreEqual(0, results.TotalRecords);
      }
   }
}